////////////////////////////////////////////////////////////// 
//File Name: string.c                                         
// Contains member and non-memberfunction definitions.
//////////////////////////////////////////////////////////////

#include "string.h"

static char *init (const char *str)
{
  int size;
  char *string = new char[size = strlen(str) + 1];

  memcpy(string, str, size);
  return string;
}


// constructor
String::String()
{
  d_string_p = init("");
}


// constructor
String::String(const String& string)
{
  d_string_p = init(string.d_string_p);
}


// constructor
String::String(const char* str)
{
  d_string_p = init(str);
}

// constructor
String::~String()
{
  delete d_string_p;
}


String & String::operator = (const String& string)
{
  return *this = string.d_string_p;
}


String & String::operator = (const char* str)
{
  int len = strlen(str);          // get length of str
  char *s = new char[len + 1];    // alloc mem for s

  strcpy(s, str);                 // copy str to s
  delete [] d_string_p;     // dealloc mem of old string
  d_string_p = s;           // reinitialize pointer
  return *this;             // return
}


String &String::operator += (const String& string)
{
  return *this += string.d_string_p;
}


String &String::operator += (const char* str)
{
  int len = length();    // get length of str1
  len += strlen(str);    // add length of str2
  char *s = new char[len + 1];   // alloc memory

  strcpy(s, d_string_p);       // copy str1 to new str
  strcat(s, str);              // concatenate strings
  delete [] d_string_p;        // dealloc mem of str1
  d_string_p = s;              // reinitialize pointer
  return *this;                // return
}


// returns length of string
int String::length() const
{
  return strlen(d_string_p);
}


String String::operator + (const String& string) const
{
  String str = *this;    // create new object

  str += string;         // concatenate strings
  return str;            // return new object
}


String String::operator + (const char* str) const
{
  String Str = *this;  // create new object
  Str += str;          // concatenate strings
  return Str;          // return new object
}



// The following functions use the standard string.h 
// library function, strcmp(), to overload operators.
// strcmp() returns 0 if the strings are equal, < 0 if
// str1 is lexicographically less than str2, and > 0
// str1 is greater than str2.  The actual return values
// are being tested in these functions.

int String::operator == (const String& string) const
{
  return ( strcmp(d_string_p, string.d_string_p) == 0 );
}


int String::operator == (const char* str) const
{
  return ( strcmp(d_string_p, str) == 0 );
}


int String::operator!=(const String& string) const {
	return ( strcmp(d_string_p, string.d_string_p) != 0 );
}


int String::operator != (const char* str) const
{
  return ( strcmp(d_string_p, str) != 0 );
}


int String::operator < (const String& string) const
{
  return ( strcmp(d_string_p, string.d_string_p) < 0 );
}


int String::operator < (const char* str) const
{
  return ( strcmp(d_string_p, str) < 0 );
}


int String::operator <= (const String& string) const
{
  return ( strcmp(d_string_p, string.d_string_p) <= 0 );
}


int String::operator <= (const char* str) const
{
  return ( strcmp(d_string_p, str) <= 0 );
}


int String::operator > (const String& string) const
{
  return ( strcmp (d_string_p, string.d_string_p) >  0 );
}


int String::operator > (const char* str) const {
  return ( strcmp (d_string_p, str) >  0 );
}


int String::operator >= (const String& string) const
{
  return ( strcmp(d_string_p, string.d_string_p) >= 0 );
}


int String::operator >= (const char* str) const
{
  return ( strcmp(d_string_p, str) >= 0 );
}


// Next functions also use the overloaded operators that
// were defined above to operate on operands of different
// types.

String operator + (const char* str, const String& string)
{
  String newstr = str;       // create new object
  newstr += string;          // concatenate strings
  return newstr;           // return object
}



// pass str as parameter to overloaded operator function
int operator == (const char* str, const String& string)
{
  return ( string.operator == (str) );
}



// pass str as parameter to overloaded operator function
int operator != (const char* str, const String& string)
{
  return ( string.operator != (str) );
}


// pass str as parameter to overloaded operator function
int operator < (const char* str, const String& string)
{
  return ( string.operator > (str) );
}


// pass str as parameter to overloaded operator function
int operator <= (const char* str, const String& string)
{
  return ( string.operator >= (str) );
}



// pass str as parameter to overloaded operator function
int operator > (const char* str, const String& string)
{
	return ( string.operator < (str) );	
}


// pass str as parameter to overloaded operator function
int operator >= (const char* str, const String& string)
{
  return ( string.operator <= (str) );
}


// overloaded I/O operator
ostream &operator << (ostream& s, String &str)
{
  // << is friend of String class
  s << str.d_string_p;  // get output string from str
  return s;             // return output
}
