//////////////////////////////////////////////////////////////////
// File Name: string.h:                                         //
// Role: contains the class definition of String, as well as    //
//       non-member overloaded operator functions.              //
//////////////////////////////////////////////////////////////////

class String
{
public:

	// Creators
	String();
	String(const String& string);
	String(const char *str);
	// Destructor
	~String();

	// Assignment
	String &operator = (const String& string);
	String &operator = (const char* str);

	// Manipulators
	String &operator += (const String& string);
	String &operator += (const char* str);
	// Concatenation (to this string)

	// Accessors
	int length() const;
	String operator+(const String& str) const;
	String operator+(const char* string) const;
	// Concatonation (to a new string)

	// Comparison Operators (lexical)
	int operator==(const String& string) const;
	int operator==(const char* str) const;
	int operator!=(const String& string) const;
	int operator!=(const char* str) const;
	int operator< (const String& string) const;
	int operator< (const char* str) const;
	int operator<=(const String& string) const;
	int operator<=(const char* str) const;
	int operator> (const String& string) const;
	int operator> (const char* str) const;
	int operator>=(const String& string) const;
	int operator>=(const char* str) const;

	// overloaded conversion operator
	operator char*()             // converts user type to standard type
	   {  return d_string_p; }   // converts String to char *

	// friend -- for overloaded I/O operator of ostream class
	friend ostream& operator << (ostream& s, String &str);

private:
	char *d_string_p;            // pointer to string of characters
};



// Non-Member (Free) Operators
String operator+(const char* str, const String& string);
int operator==(const char* str, const String& string);
int operator!=(const char* str, const String& string);
int operator> (const char* str, const String& string);
int operator>=(const char* str, const String& string);
int operator< (const char* str, const String& string);
int operator<=(const char* str, const String& string);