// Definicje funkcji skladowych klasy Tablica

#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <assert.h>
#include "tablica1.h"

// Inicjacja statycznej danej skladowej w zasiegu pliku
int Tablica::liczbaTablic = 0; // nie ma jeszcze obiektow

// konstruktor domyslny klasy Tablica  (domyslny rozmiar 10)
Tablica::Tablica(int rozmiarTablicy)
{
  rozmiar = (rozmiarTablicy >  0) ? rozmiarTablicy : 10;
  ptr = new int[rozmiar]; // zarezerwuj miejsce na tablice
  assert(ptr != 0);       // zakoncz, jezeli nie zaalokowano
  ++liczbaTablic;         // dodaj kolejny obiekt

  for(int i = 0; i < rozmiar; i++)
   ptr[i] = 0;            // zainicjowanie tablicy
}

// konstruktor kopiujacy klasy Tablica
// musi otrzymac referencje, aby uniknac nieskonczonego wywolywania
Tablica::Tablica(const Tablica &inic) : rozmiar(inic.rozmiar)
{   
  ptr = new int[rozmiar];   // zarezerwuj miejsce na tablice
  assert(ptr != 0);        // zakoncz, jezeli nie zaalokowano 
  ++liczbaTablic;          // dodaj kolejny obiekt

  for(int i = 0; i < rozmiar; i++)
   ptr[i] = inic.ptr[i];            // kopiowanie inic do obiektu     
}

// Destruktor klasy Tablica
Tablica::~Tablica()
{
  delete [] ptr;                    // zwolnij pamiec tablicy
  --liczbaTablic;                   // jeden obiekt mniej
}

// Pobierz rozmiar tablicy
int Tablica::pobierzRozmiar() const { return rozmiar; }

// Przeciazony operator przypiania
// zwracnie stalej unika: (t1 = t2) = t3
const Tablica &Tablica::operator=(const Tablica &prawa)
{
  if( &prawa != this ) { // sprawdz, czy przypisanie samej sobie
    // jezeli tablice maja rozne wielkosci, zwolnij pamiec dla
    // tablicy z lewej strony, nastepnie zaalokuj ja jeszcze raz
    if(rozmiar != prawa.rozmiar){
     delete [] ptr;              // zwolnij pamiec tablicy
     rozmiar = prawa.rozmiar;    // zmien wielkosc tablicy
     ptr     = new int[rozmiar]; // zaalokuj pamiec na nowe tablice
     assert(ptr != 0);            // zakoncz, jezeli nie zaalokowano
    }
    
    for(int i = 0; i < rozmiar; i++)
        ptr[i] = prawa.ptr[i];   // skopiowanie danych z tablicy
   }

   return *this;                 // umozliwia z = y = z
}

// Sprawdz, czy dwie tablice sa rowne i zwroc true,
// w przeciwnym przypadku zwroc false
bool Tablica::operator==(const Tablica &prawa) const
{
  if(rozmiar != prawa.rozmiar)
   return 0;          // tablice maja rozne wielkosci

  for( int i = 0; i < rozmiar; i++)
   if(ptr[i] != prawa.ptr[i])
    return 0;         // tablice sa rozne

  return 1;            // tablice sa rowne
}

// Przeciazony operator indeksu dla tablic 
// zwrocenie referencji tworzy l-value
int  &Tablica::operator[](int indeks)
{
 // Sprawdz, czy indeks ma odpowiednia wartosc
 assert(0 <= indeks && indeks < rozmiar);

 return ptr[indeks];   // zwrocenie referencji
}                    

// Przeciazony operator indeksu dla tablic 
// zwrocenie stalej referencji tworzy r-value
const int  &Tablica::operator[](int indeks) const
{
 // Sprawdz, czy indeks ma odpowiednia wartosc
 assert(0 <= indeks && indeks < rozmiar);

 return ptr[indeks];   // zwrocenie referencji
}                    

// Zwraca liczbe utworzonych obiektow klasy Tablica
// Funkcja statyczna nie moze byc stala
int Tablica::pobierzLiczbeTablic() { return liczbaTablic; }

// Przeciazony operator wejscia dla klasy Tablica
istream &operator>>(istream &wejscie, Tablica &t)
{
 for(int i = 0; i < t.rozmiar; i++)
  wejscie >> t.ptr[i];

 return wejscie;   // umozliwia >> x >> y;
}

// Przeciazony operator wyjscia dla klasy Tablica
ostream &operator<<(ostream &wyjscie, const Tablica &t)
{
  int i;

  for(i = 0; i < t.rozmiar; i++){

    wyjscie << setw(12) << t.ptr[i];

    if ((i + 1) % 4 == 0)  // wyswitlenie 4 liczb w wierszu
     wyjscie << endl;
  }

  if (i % 4 != 0)
   wyjscie << endl;

  return wyjscie;  // umozliwia cout << x << y;
}
      
      
     
    
