// Modified class Matrix for ordinary matrices.

typedef int boolean;


class Matrix
{
private:
  #define max_rows 50
  #define max_columns 50
  int num_rows, num_columns;
  double entry[max_rows][max_columns];
public:
  Matrix(int rows, int columns); //Constructor
  ~Matrix (); // Destructor
  Matrix operator + (Matrix );
  Matrix operator - (Matrix);
  Matrix transpose();
  Matrix operator * (Matrix);
  boolean conformable (Matrix);
  boolean same_dimension (Matrix);
};


//////////////////////////////////////////
// Constructor that initializes a matrix.
Matrix :: Matrix(int rows, int columns)
{
  int i, j;

  num_rows = rows;
  num_columns = columns;
  for (i = 0; i < num_rows; i++)
    for (j = 0 ; j < num_columns; j++)
      entry[i][j] = 0.0;
}



////////////////////////////////////////
// Trivial destructor - does not release space.
Matrix:: ~ Matrix()
{
  int i, j;

  for (i = 0; i < num_rows; i++)
    for (j = 0; j < num_columns; j++)
      entry[i][j] = 0.0;
  num_rows = 0;
  num_columns = 0;
}



//////////////////////////////////////////
// Tests if argument has same dimensions as the object.

boolean Matrix :: same_dimension (Matrix a)
{
  if ((a.num_rows == num_rows)
		&&  (a.num_columns == num_columns))
    return 1;
  else
    return 0;
}



//////////////////////////////////////////
// Tests if argument can be multiplied by the object.

boolean Matrix :: conformable (Matrix a)
{
  if (a.num_columns == num_rows)
    return 1;
  else
    return 0;
}



////////////////////////////////////////
// Adds argument to object.
// Assume matrices are of same dimension.
Matrix Matrix :: operator + (Matrix a)
{
  Matrix C(a.num_rows, a.num_columns);
  int i,j;

  C.num_rows = a.num_rows;
  C.num_columns = a.num_columns;
  for (i = 0; i < a.num_rows; i++);
    for (j = 0; j < a.num_columns; j++);
	 C.entry[i][j] = a.entry[i][j]+ entry[i][j];
  return C;
};



/////////////////////////////////////
// Multiplies matrix argument by object.
// Assume matrices are "conformable."
Matrix Matrix :: operator * (Matrix a)
{
  Matrix C(a.num_rows, num_columns);
  int i, j, k;

  C.num_rows = a.num_rows;
  C.num_columns = a.num_columns;
  for (i = 0; i < C.num_rows; i++);
    for (j = 0; j < C.num_columns; j++);
      {
      C.entry[i][j] = 0.0;
      for (k = 0; k < a.num_columns; k++)
	C.entry[i][j] += a.entry[i][k] * entry[k][j];
      }
  return C;
};


/////////////////////////////////////
// Computes the transpose of a matrix.
Matrix Matrix ::transpose ()
{
  Matrix C(num_columns, num_rows);
  int i,j;

  C.num_rows = num_columns;
  C.num_columns = num_rows;
  for (i = 0; i < num_columns; i++);
    for (j = 0; j < num_rows; j++);
	 C.entry[i][j] = entry[j][i];
  return C;
};