// File ex5_10.h
//
// This file contains three class descriptions.
// The first is for the class List_Node, which serves
// as the nodes of the linked list.
// The next class described here is the list itself.
// The final class described here is an iterator class.
// All three classes use templates.
//

// Define the class of nodes for the linked list class.

template <class T> class List_Node
{
public:
  // Constructor
  List_Node (T, List_Node *);
  List_Node * insert(T);
  ~List_Node();
  T get_data();
private:
  T info;
  List_Node * next;
  List_Node * copy();
protected:
  friend class List<T>;
  friend class List_Iterator<T>;
//  friend class Queue<T>;
};



// Constructor function for class List_Node.
template<class T> List_Node<T> ::List_Node(T val, List_Node<T> *p)

{
  info = val;
  next = p;
}


// Destructor.
template <class T> List_Node<T> :: ~List_Node()
{
  delete this;
}



// Examine the contents of a node.
template <class T> T List_Node<T> :: get_data()
{
  return info;
}




////////////////////////////////////////////
// Define the class List that uses the nodes
// defined in the class List_Node.
////////////////////////////////////////////
template <class T> class List
{
public:
  // Constructor
  List()  {head = 0;} ;
  // Copy constructor
  List(const List & );
  //Destructor
  ~List();

  void remove_first();
  void insert(T value);
  void clear();
  int empty() const;
  T get_data();
  List_Node<T> * head;
protected:
  friend class List<T>;
  friend List_Iterator<T>;
//  friend class Queue<T>;
};



// Destructor.
template <class T> List<T> :: ~List()
{
  List_Node<T> * temp = head;
  while (temp != 0)
    {
    head = temp;
    delete temp;
    temp = head->next;
    }
  head = 0;
}



// Makes a copy of each List_Node by following pointers.
template <class T> List_Node<T> * List_Node<T>:: copy()
{
  List_Node<T> * temp;

  if (next != 0)
    temp = new List_Node<T>(info, next->copy());
  else
    temp = new List_Node<T>(info, 0);

  return temp;
}




///////////////////////////////////////////
// Define the iterator class.
///////////////////////////////////////////
template <class T> class List_Iterator
{
public:
  List_Iterator( List<T> &);
  void init();

  virtual int operator ++();
  void remove();
  void insert_before(T val);
  void insert_after(T val);
  List_Node <T> *forward;
  List_Node<T> *backward;
protected:
  List<T> & List_Head;
};




// Constructor for List_Iterator class. It uses an
// initializer and then calls a function init.
template <class T> List_Iterator<T>:: List_Iterator(
		List<T> &list1): List_Head(list1)
{
}



template <class T> void List_Iterator<T>:: init()
{
  backward = 0;
  forward = List_Head.head;
}


// Operator ++ to indicate moving through the nodes of
// a list one element at at time.
#include <iostream.h>

template <class T> int List_Iterator<T>:: operator ++ ()
{
  if (forward == 0)
    {
    if (backward == 0)
      forward = List_Head.head;
    else
      forward = backward->next;
    }
  else
    {
    cout << forward->info << ' ' ;
    backward = forward;
    forward = forward->next;
    }
  return (forward !=0);
}



//Remove current node from list.
template <class T> void List_Iterator<T>:: remove()
{
  if (backward == 0)
    List_Head.head = forward->next;
  else
    backward->next = forward->next;

  delete forward;
}




// Function to insert a node before the current node.
template <class T> void List_Iterator<T>::
				 insert_before(T value)
{
  // Insertion before the start of list.
  if (!backward)
    {
    List_Head.List<T>::insert(value);
    backward = List_Head.head;
    forward = backward->next;
    }
  else
    backward = backward->next;
}



// Function to insert after a node.
template <class T> void List_Iterator<T>::
				insert_after(T value)
{
  // Insertion in normal situation.
  if (forward)
    forward->insert(value);

  // Insertion at end of list.
  else if (backward)
    forward = backward->insert(value);

  else // Insert at start list.
    List_Head.List<T>:: insert(value);
}
