// File: ex5_12.h
//
// This file contains the class description for the
// Sparse_Matrix class.
//

#include <iostream.h>

typedef int boolean;


// Class Sparse_Matrix
class Sparse_Matrix
{
private:
  int num_rows, num_columns;
  struct Entry
    {
    int row, column;
    float value;
    Entry * next;
    } *entry;
  Entry * entry_start;
public:
  // Constructor function.
  Sparse_Matrix(int rows, int columns);
  // Destructor function.
  ~Sparse_Matrix();

  void init(int row, int column, float value);
  Sparse_Matrix operator + (Sparse_Matrix);
  boolean conformable (Sparse_Matrix);
  boolean same_dimension (Sparse_Matrix);
};



// Constructor function for the class Sparse_Matrix.
Sparse_Matrix :: Sparse_Matrix(int rows, int columns)
{
  num_rows = rows;
  num_columns = columns;
  entry_start = new Entry;
  entry_start->row = 0;
  entry_start->column = 0;
  entry_start->value = 0;
  entry_start->next = 0;
}




////////////////////////////////////////
// Trivial destructor - does not release space.
Sparse_Matrix:: ~ Sparse_Matrix()
{
  num_rows = 0;
  num_columns = 0;
  delete entry_start;
}




////////////////////////////////////////////////
// Initializes the contents of the given row and
// column to the third argument.
////////////////////////////////////////////////
void Sparse_Matrix::init(int row, int column, float val)
{
  struct Entry *temp;

  temp = entry_start;
  while (temp != 0)
    if ((temp->row != row) && (temp->column != column))
      temp = temp->next;
    // If found, assign value to this row and column.
    if (temp != 0)
      temp->value = val;
    else // insert after start for lack of better place.
      {
      temp = new Entry;
      temp->row = row;
      temp->column = column;
      temp->value = val;
      temp->next = entry_start->next;
      entry_start->next = temp;
      }
cout << temp->row << ' '<< temp->column<< ' ' << val;
}



//////////////////////////////////////////////////////
// Tests if argument has same dimensions as the object.

boolean Sparse_Matrix :: same_dimension (Sparse_Matrix a)
{
  if ((a.num_rows == num_rows)
		&&  (a.num_columns == num_columns))
    return 1;
  else
    return 0;
}



////////////////////////////////////////////////////
// Tests if argument can be multiplied by the object.
// NOTE: no multiplication is done member functions.
////////////////////////////////////////////////////
boolean Sparse_Matrix :: conformable (Sparse_Matrix a)
{
  if (a.num_columns == num_rows)
    return 1;
  else
    return 0;
}



////////////////////////////////////////
// Adds argument to object.
// Assume matrices are of same dimension.
Sparse_Matrix Sparse_Matrix :: operator + (Sparse_Matrix a)
{
  Sparse_Matrix C(a.num_rows, a.num_columns);
  int i,j;
  struct Entry *temp_a, *temp_b, *temp_c, *temp;

  C.num_rows = a.num_rows;
  C.num_columns = a.num_columns;
  temp_a = a.entry_start; // first arg's entry_start
  temp_b = entry_start ; // object's entry_start
  temp_c = C.entry_start;   // return's entry_start

  while (temp_b != 0)
    {
    // Assign object's nonzero entries to C.
    temp_c->value = temp_b->value;
    temp_c->row = temp_b->row;
    temp_c->column = temp_b->column;
    temp_c->next = temp_b->next;
    temp_b = temp_b->next;
    temp_c = temp_c->next;
    }

  // Reinitialize pointer to C's data.
  temp_c = C.entry_start;
  // Now add elements of argument's matrix to C.
  while (temp_a != 0)
    {
    // Use linear search on C.
    temp_c = C.entry_start;
    while ((temp_c->row != temp_a->row) &&
		(temp_c->column != temp_a->column) &&
		(temp_c != 0))
      temp_c = temp_c->next;

    if (temp_c != 0) // Found a match of row, column
      temp_c->value += temp_b->value;
    else // Make a new entry in C.
      {
      temp = new Entry;
      temp->row = temp_a->row;
      temp->column = temp_a->column;
      temp->value = temp_a->value;
      temp->next = C.entry_start->next;
      C.entry_start->next = temp;
      }
    temp_a = temp_a->next;
    }
  return C;
}