// FILE ex5_17.h
//
// Description of non-template-based Tree class.
//

#include <iostream.h>
#include <iomanip.h>

typedef int Item;


class Tree_Node
{
friend class Tree;

public:
  Tree_Node(const Item &);
  Item process();
private:
  Tree_Node *left;
  Item info;
  Tree_Node *right;
};


// Constructor function for Tree_Node, which will
// hold nodes of Tree.
Tree_Node:: Tree_Node(const Item &data)
{
  info = data;
  left = 0;
  right = 0;
}



// Get information stored in Tree_Node. This simple
// function just prints the contents of the node.
Item Tree_Node:: process()
{
  cout << setw(4) << info;
  return info;
}



///////////////////////////////////////////
// Definition of class Tree.
///////////////////////////////////////////

class Tree
{
public:
  Tree();
  void insert(const Item &);

  // Traversal functions available to outside world.
  void preorder();
  void postorder();
  void inorder();

private:
  Tree_Node * root;
  void insert_Node(Tree_Node ** , const Item &);

  // Traversal functions available only to member functions.
  void preorder_Node(Tree_Node *);
  void postorder_Node(Tree_Node *);
  void inorder_Node(Tree_Node *);
};



// Constructor function for class Tree without templates.
Tree ::Tree()
{
  root = 0;
}



// Insertion function for trees using insert_Node
// function for trees.
void Tree :: insert(const Item & val)
{
  insert_Node(&root, val);
}



// Lower-level node insertion function to insert
// data into the tree in "sorted order," assuming
// that the relational operators < and > are properly
// overloaded for the Item data type.

void Tree:: insert_Node(Tree_Node **p, const Item & val)
{
  if (*p == 0)  // Node set is empty.
    {
    *p = new Tree_Node(val);
    (*p)->left = (*p)->right = 0;
    }
  else
    {
    if (val < (*p)->info)
      insert_Node(& ((*p)->left), val);
    else // value is greater than or equal to contents
      insert_Node(& ((*p)->right), val);
    }
}



////////////////////////////////////////////////////////
// Tree traversal functions.  For each type of traversal
// (inorder, preorder, postorder), the higher-level
// function is followed by a lower-level function for
// doing the actual work of the traversal.
////////////////////////////////////////////////////////

void Tree :: preorder()
{
  preorder_Node(root);
}


void Tree :: preorder_Node(Tree_Node *p)
{
  if (p)
    {
    (*p).process();
    preorder_Node(p->left);
    preorder_Node(p->right);
    }
}

////////////////////////////

void Tree :: postorder()
{
  postorder_Node(root);
}


void Tree :: postorder_Node(Tree_Node *p)
{
  if (p)
    {
    postorder_Node(p->left);
    postorder_Node(p->right);
    (*p).process();
    }
}

////////////////////////////

void Tree :: inorder()
{
  inorder_Node(root);
}


void Tree :: inorder_Node(Tree_Node *p)
{
  if (p)
    {
    inorder_Node(p->left);
    (*p).process();
    inorder_Node(p->right);
    }
}