// FILE ex5_19.h
//
// Contains class definitions for nodes and trees
// using templates.
//

#include <iostream.h>
#include <iomanip.h>


//////////////////////////////////
// Definition of class Tree_Node.
//////////////////////////////////

template<class T> class Tree_Node
{
friend class Tree<T>;

public:
  Tree_Node(const T &);
  T process();
private:
  Tree_Node *left;
  T info;
  Tree_Node *right;
};



// Constructor function for Tree_Node, which will
// hold parameterized nodes of Tree.
template<class T> Tree_Node<T>:: Tree_Node(const T &data)
{
  info = data;
  left = 0;
  right = 0;
}



// Get information stored in Tree_Node. This simple
// function just prints the contents of the node.
template<class T> T Tree_Node<T>:: process()
{
  cout << setw(4) << info;
  return info;
}



///////////////////////////////////////////
// Definition of class Tree.
///////////////////////////////////////////

template <class T> class Tree
{
public:
  Tree();
  void insert(const T &);

  // Traversal functions available to outside world.
  void preorder();
  void postorder();
  void inorder();

private:
  Tree_Node <T> * root;
  void insert_Node(Tree_Node<T>**, const T &);

  // Traversal functions available only to member functions.
  void preorder_Node(Tree_Node<T> *);
  void postorder_Node(Tree_Node<T> *);
  void inorder_Node(Tree_Node<T> *);
};



// Constructor function for class Tree using templates.
template<class T> Tree<T> ::Tree()
{
  root = 0;
}



// Insertion function for trees using insert_Node
// function for trees.
template<class T> void Tree<T> :: insert(const T & val)
{
  insert_Node(&root, val);
}



// Lower-level node insertion function to insert
// data into the tree in "sorted order," assuming
// that the relational operators < and > are properly
// overloaded for this template class.

template<class T> void Tree<T> :: insert_Node(
				Tree_Node <T> **p,
				const T & val)
{
  if (*p == 0)  // Node set is empty.
    {
    *p = new Tree_Node<T>(val);
    (*p)->left = (*p)->right = 0;
    }
  else
    {
    if (val < (*p)->info)
      insert_Node(& ((*p)->left), val);
    else // value is greater than or equal to contents
      insert_Node(& ((*p)->right), val);
    }
}



////////////////////////////////////////////////////////
// Tree traversal functions.  For each type of traversal
// (inorder, preorder, postorder), the higher-level
// function is followed by a lower-level function for
// doing the actual work of the traversal.
////////////////////////////////////////////////////////

template<class T> void Tree<T> :: preorder()
{
  preorder_Node(root);
}


template<class T> void Tree<T> :: preorder_Node(Tree_Node<T> *p)
{
  if (p)
    {
    (*p).process();
    preorder_Node(p->left);
    preorder_Node(p->right);
    }
}

////////////////////////////

template<class T> void Tree<T> :: postorder()
{
  postorder_Node(root);
}


template<class T> void Tree<T> :: postorder_Node(Tree_Node<T> *p)
{
  if (p)
    {
    postorder_Node(p->left);
    postorder_Node(p->right);
    (*p).process();
    }
}

////////////////////////////

template<class T> void Tree<T> :: inorder()
{
  inorder_Node(root);
}


template<class T> void Tree<T> :: inorder_Node(Tree_Node<T> *p)
{
  if (p)
    {
    inorder_Node(p->left);
    (*p).process();
    inorder_Node(p->right);
    }
}